@@ -202,6 +202,9 @@ panel_application_init (PanelApplication *application)
   application->drop_occurred = FALSE;
   application->autohide_block = 0;
 
+  /* set application name so internal plugins can't override it */
+  g_set_application_name (PACKAGE_NAME);
+
   /* get the xfconf channel (singleton) */
   application->xfconf = xfconf_channel_get (XFCE_PANEL_CHANNEL_NAME);
 
@@ -867,7 +870,8 @@ panel_application_plugin_insert (PanelApplication *application,
                         GTK_WIDGET (provider), position);
 
   /* send all the needed info about the panel to the plugin */
-  panel_window_set_provider_info (window, provider, FALSE);
+  if (gtk_widget_get_realized (GTK_WIDGET (window)))
+    panel_window_set_provider_info (window, provider, FALSE);
 
   /* show the plugin */
   gtk_widget_show (provider);
@@ -1626,7 +1630,11 @@ panel_application_new_window (PanelApplication *application,
   if (new_window)
     panel_application_save (application, SAVE_PANEL_IDS);
 
-  gtk_widget_show (GTK_WIDGET (window));
+  /*
+   * Do not show the panel window here, this will trigger allocation and/or critical
+   * warnings if the output to which it is assigned is disconnected. It will be shown
+   * in panel_window_screen_layout_changed() if necessary.
+   */
 
   return PANEL_WINDOW (window);
 }
