@@ -221,15 +221,24 @@ panel_module_load (GTypeModule *type_module)
       if (make_resident)
         g_module_make_resident (module->library);
     }
-  else if (!g_module_symbol (module->library, "xfce_panel_module_construct",
-                             (gpointer) &module->construct_func))
+  else
     {
-      g_critical ("Module \"%s\" lacks a plugin register function.",
-                  module->filename);
-
-      panel_module_unload (type_module);
-
-      return FALSE;
+      if (g_module_symbol (module->library, "xfce_panel_module_construct",
+                           (gpointer) &module->construct_func))
+        {
+          /* avoid static memory allocation issues when removing the plugin, with e.g
+           * g_intern_static_string() or G_PARAM_STATIC_STRINGS in signals/properties
+           * declarations, and keep plugin types in memory so plugin can be re-added
+           * after being removed */
+          if (module->mode == PANEL_MODULE_RUN_MODE_INTERNAL)
+            g_module_make_resident (module->library);
+        }
+      else
+        {
+          g_critical ("Module \"%s\" lacks a plugin register function.", module->filename);
+          panel_module_unload (type_module);
+          return FALSE;
+        }
     }
 
   return TRUE;
@@ -342,9 +351,10 @@ panel_module_new_from_desktop_file (const gchar *filename,
           /* run mode of the module, by default everything runs in
            * the wrapper, unless defined otherwise or unsupported */
           if (forced_mode != PANEL_MODULE_RUN_MODE_INTERNAL
-              && (WINDOWING_IS_X11 () || gtk_layer_is_supported ())
-              && (forced_mode == PANEL_MODULE_RUN_MODE_EXTERNAL
-                  || !xfce_rc_read_bool_entry (rc, "X-XFCE-Internal", FALSE)))
+              && ((WINDOWING_IS_X11 ()
+                   && (forced_mode == PANEL_MODULE_RUN_MODE_EXTERNAL
+                       || !xfce_rc_read_bool_entry (rc, "X-XFCE-Internal", FALSE)))
+                  || (gtk_layer_is_supported () && forced_mode == PANEL_MODULE_RUN_MODE_EXTERNAL)))
             {
               module->mode = PANEL_MODULE_RUN_MODE_EXTERNAL;
               g_free (module->api);
@@ -446,16 +456,10 @@ panel_module_new_plugin (PanelModule *module,
               debug_type = "construct-func";
             }
 
-          if (G_LIKELY (plugin != NULL))
-            break;
-          else
+          if (G_UNLIKELY (plugin == NULL))
             g_type_module_unuse (G_TYPE_MODULE (module));
         }
-
-      /* fall-through (make wrapper plugin), probably a plugin with
-       * preinit_func which is not supported for internal plugins
-       * note: next comment tells GCC7 to ignore the fallthrough */
-      /* fall through */
+      break;
 
     case PANEL_MODULE_RUN_MODE_EXTERNAL:
       plugin = panel_plugin_external_wrapper_new (module, unique_id, arguments);
